#include "Process.h"

/**
* La nostra unica reference a Process. La gestione della concorrenza viene fatta internamente
*/
Process* Process::Instance = NULL;
UINT Process::uReference = 0;
list<WindowEntry>* Process::pWindowList = NULL;

Process* Process::self()
{
	if(Instance == NULL)
		Instance = new Process();

	uReference++;

	return Instance;
}

Process::Process() : hSnap(INVALID_HANDLE_VALUE), hProcessMutex(INVALID_HANDLE_VALUE), 
hWindowMutex(INVALID_HANDLE_VALUE), pList(NULL){
	memset(&pe, 0x00, sizeof(pe));
	memset(&procEntry, 0x00, sizeof(procEntry));

	ulTime = 0;
	ulWinTime = 0;

	hProcessMutex = CreateMutex(NULL, FALSE, NULL);
	hWindowMutex = CreateMutex(NULL, FALSE, NULL);
}

Process::~Process() {
	uReference--;

	if(uReference)
		return;

	if(pList) {
		pList->clear();
		delete pList;
	}

	if(pWindowList) {
		pWindowList->clear();
		delete pWindowList;
	}

	if(hSnap != INVALID_HANDLE_VALUE)
		CloseToolhelp32Snapshot(hSnap);

	if(hProcessMutex != INVALID_HANDLE_VALUE)
		CloseHandle(hProcessMutex);

	if(hWindowMutex != INVALID_HANDLE_VALUE)
		CloseHandle(hWindowMutex);
}

UINT Process::GetPid(WCHAR *pProcName) {
	if(RefreshProcessList() == FALSE)
		return FALSE;

	WAIT_AND_SIGNAL(hProcessMutex);

	if(pList == NULL || pProcName == NULL || wcslen(pProcName) == 0) {
		UNLOCK(hProcessMutex);
		return 0;
	}

	for(iter = pList->begin(); iter != pList->end(); iter++) {
		if(_wcsnicmp(pProcName, (*iter).pe.szExeFile, MAX_PATH) == 0) {
			UNLOCK(hProcessMutex);
			return (*iter).pe.th32ProcessID;
		}
	}

	UNLOCK(hProcessMutex);
	return 0;
}

BOOL Process::IsProcessRunning(WCHAR *pProcName) {
	if(GetPid(pProcName))
		return TRUE;

	return FALSE;
}

BOOL Process::RefreshProcessList() {
	ProcessEntry tProcEntry;
	unsigned __int64 utTime;

	WAIT_AND_SIGNAL(hProcessMutex);

	utTime = GetTime();

	// Refreshiamo la lista solo se e' passato piu' di un secondo
	if(DiffTime(utTime, ulTime) < 1000) {
		UNLOCK(hProcessMutex);
		return TRUE;
	}

	ulTime = utTime;

	pe.dwSize = sizeof(pe);

	if(hSnap != INVALID_HANDLE_VALUE) {
		CloseToolhelp32Snapshot(hSnap);
		hSnap = INVALID_HANDLE_VALUE;
	}

	if(pList) {
		pList->clear();
		delete pList;
		pList = NULL;		
	}

	pList = new(std::nothrow) list<ProcessEntry>;

	if(pList == NULL) {
		UNLOCK(hProcessMutex);
		return FALSE;
	}

	// Il secondo flag e' un undocumented che serve a NON richiedere la lista
	// degli heaps altrimenti la funzione fallisce sempre per mancanza di RAM.
	hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS | TH32CS_SNAPNOHEAPS, 0);
	
	if(hSnap == INVALID_HANDLE_VALUE) {
		UNLOCK(hProcessMutex);
		return FALSE;
	}

	if(Process32First(hSnap, &pe) == FALSE) {
		if(GetLastError() == ERROR_NO_MORE_FILES) {
			UNLOCK(hProcessMutex);
			return TRUE;
		}

		UNLOCK(hProcessMutex);
		return FALSE;
	}

	tProcEntry.bTriggered = FALSE;
	memcpy(&tProcEntry.pe, &pe, sizeof(pe));

	pList->push_back(tProcEntry);

	while(Process32Next(hSnap, &pe)) {
		tProcEntry.bTriggered = FALSE;
		memcpy(&tProcEntry.pe, &pe, sizeof(pe));

		pList->push_back(tProcEntry);
	}

	if(GetLastError() == ERROR_NO_MORE_FILES) {
		UNLOCK(hProcessMutex);
		return TRUE;
	}

	UNLOCK(hProcessMutex);
	return FALSE;
}

BOOL Process::RefreshWindowList() {
	unsigned __int64 utTime;

	WAIT_AND_SIGNAL(hWindowMutex);

	utTime = GetTime();

	// Refreshiamo la lista solo se e' passato piu' di un secondo
	if(DiffTime(utTime, ulWinTime) < 1000) {
		UNLOCK(hWindowMutex);
		return TRUE;
	}

	ulTime = utTime;

	if(pWindowList) {
		pWindowList->clear();
		delete pWindowList;
		pWindowList = NULL;		
	}

	pWindowList = new(std::nothrow) list<WindowEntry>;

	if(pWindowList == NULL) {
		UNLOCK(hWindowMutex);
		return FALSE;
	}

	if(EnumWindows(&EnumCallback, NULL) == FALSE) {
		UNLOCK(hWindowMutex);
		return FALSE;
	}

	UNLOCK(hWindowMutex);
	return TRUE;
}

BOOL Process::IsWindowPresent(WCHAR *pWindowName) {
	if(RefreshWindowList() == FALSE)
		return FALSE;

	WAIT_AND_SIGNAL(hWindowMutex);

	if(pWindowList == NULL || pWindowName == NULL || wcslen(pWindowName) == 0) {
		UNLOCK(hWindowMutex);
		return FALSE;
	}
 
	for(iterWindow = pWindowList->begin(); iterWindow != pWindowList->end(); iterWindow++) {
		if(CmpWildW(pWindowName, (*iterWindow).wWindowTitle) != 0) {
			UNLOCK(hWindowMutex);
			return TRUE;
		}
	}

	UNLOCK(hWindowMutex);
	return FALSE;
}

// Compara due stringhe con wildcard, torna 0 se le stringhe sono diverse
INT Process::CmpWildW(WCHAR *wild, WCHAR *string) {
	WCHAR *cp = NULL, *mp = NULL;

	while ((*string) && (*wild != '*')) {
		if ((towupper((WCHAR)*wild) != towupper((WCHAR)*string)) && (*wild != '?')) {
			return 0;
		}
		wild++;
		string++;
	}

	while (*string) {
		if (*wild == '*') {
			if (!*++wild) {
				return 1;
			}

			mp = wild;
			cp = string+1;
		} else if ((towupper((WCHAR)*wild) == towupper((WCHAR)*string)) || (*wild == '?')) {
			wild++;
			string++;
		} else {
			wild = mp;
			string = cp++;
		}
	}

	while (*wild == '*') {
		wild++;
	}

	return !*wild;
}

BOOL CALLBACK Process::EnumCallback(HWND hwnd, LPARAM lParam) {
	WindowEntry tWindowEntry;

	tWindowEntry.bTriggered = FALSE;
	
	if(GetWindowText(hwnd, tWindowEntry.wWindowTitle, MAX_TITLE_LEN) && tWindowEntry.wWindowTitle[0] != 0) {
		pWindowList->push_back(tWindowEntry);
	}

	return TRUE;
}